/** Header file generated with fdesign on Mon Mar 23 00:21:56 1998.**/

#ifndef FD_QuickCam_h_
#define FD_QuickCam_h_

/** Callbacks, globals and object handlers **/
extern void Slider(FL_OBJECT *, long);
extern void Quit(FL_OBJECT *, long);
extern void About(FL_OBJECT *, long);
extern void Graph(FL_OBJECT *, long);
extern void automode(FL_OBJECT *, long);
extern void XQC_Param(FL_OBJECT *, long);
extern void XQC_Browser(FL_OBJECT *, long);
extern void XQC_Button(FL_OBJECT *, long);
extern void Stop(FL_OBJECT *, long);
extern void Help(FL_OBJECT *, long);
extern void LoadSave(FL_OBJECT *, long);
extern void XQC_graph1(FL_OBJECT *, long);

extern void AboutOk(FL_OBJECT *, long);

extern void Histogram(FL_OBJECT *, long);


/**** Forms and Objects ****/
typedef struct {
	FL_FORM *QuickCam;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *Contrast;
	FL_OBJECT *Brightness;
	FL_OBJECT *WhiteBal;
	FL_OBJECT *Quit;
	FL_OBJECT *About;
	FL_OBJECT *FD_fps;
	FL_OBJECT *FD_rez;
	FL_OBJECT *Graph;
	FL_OBJECT *autobutt;
	FL_OBJECT *FD_resolution;
	FL_OBJECT *FD_bpp;
	FL_OBJECT *FD_image1;
	FL_OBJECT *FD_image2;
	FL_OBJECT *FD_image4;
	FL_OBJECT *FD_image5;
	FL_OBJECT *FD_image3;
	FL_OBJECT *FD_image6;
	FL_OBJECT *FD_PARAM_1;
	FL_OBJECT *FD_PARAM_2;
	FL_OBJECT *FD_PARAM_3;
	FL_OBJECT *FD_BROWSER_1;
	FL_OBJECT *FD_DELETE_1;
	FL_OBJECT *FD_BROWSER_2;
	FL_OBJECT *FD_BROWSER_3;
	FL_OBJECT *FD_BROWSER_4;
	FL_OBJECT *FD_BROWSER_5;
	FL_OBJECT *FD_DELETE_2;
	FL_OBJECT *FD_DELETE_3;
	FL_OBJECT *FD_DELETE_4;
	FL_OBJECT *FD_DELETE_5;
	FL_OBJECT *FD_LOG_BROWSER;
	FL_OBJECT *Stop;
	FL_OBJECT *Help;
	FL_OBJECT *Save;
	FL_OBJECT *Load;
	FL_OBJECT *FD_graph1;
	FL_OBJECT *FD_graph2;
	FL_OBJECT *FD_graph3;
	FL_OBJECT *FD_graph4;
	FL_OBJECT *FD_graph5;
} FD_QuickCam;

extern FD_QuickCam * create_form_QuickCam(void);
typedef struct {
	FL_FORM *AboutBox;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *AboutText;
} FD_AboutBox;

extern FD_AboutBox * create_form_AboutBox(void);
typedef struct {
	FL_FORM *HGram;
	void *vdata;
	char *cdata;
	long  ldata;
	FL_OBJECT *Histogram;
} FD_HGram;

extern FD_HGram * create_form_HGram(void);

#endif /* FD_QuickCam_h_ */
