/* Form definition file generated with fdesign. */

#include <forms.h>
#include <stdlib.h>
#include "controlpanel.h"

FD_QuickCam *create_form_QuickCam(void)
{
  FL_OBJECT *obj;
  FD_QuickCam *fdui = (FD_QuickCam *) fl_calloc(1, sizeof(*fdui));

  fdui->QuickCam = fl_bgn_form(FL_NO_BOX, 715, 500);
  obj = fl_add_box(FL_FRAME_BOX,0,0,715,500,"");
  obj = fl_add_text(FL_NORMAL_TEXT,235,30,175,15,"Image Processing for Robot Vision");
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->Contrast = obj = fl_add_valslider(FL_VERT_NICE_SLIDER,80,85,30,130,"Cnt");
    fl_set_object_lalign(obj,FL_ALIGN_TOP);
    fl_set_object_callback(obj,Slider,2);
  fdui->Brightness = obj = fl_add_valslider(FL_VERT_NICE_SLIDER,20,85,30,130,"Bri");
    fl_set_object_lalign(obj,FL_ALIGN_TOP);
    fl_set_object_callback(obj,Slider,0);
  fdui->WhiteBal = obj = fl_add_valslider(FL_VERT_NICE_SLIDER,50,85,30,130,"WBal");
    fl_set_object_lalign(obj,FL_ALIGN_TOP);
    fl_set_object_callback(obj,Slider,1);
  fdui->Quit = obj = fl_add_button(FL_NORMAL_BUTTON,615,55,80,20,"Quit");
    fl_set_object_callback(obj,Quit,0);
  fdui->About = obj = fl_add_button(FL_NORMAL_BUTTON,535,15,80,20,"About");
    fl_set_object_callback(obj,About,0);
  fdui->FD_fps = obj = fl_add_text(FL_NORMAL_TEXT,100,15,90,20,"Text");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->FD_rez = obj = fl_add_text(FL_NORMAL_TEXT,100,35,90,20,"Text");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->Graph = obj = fl_add_lightbutton(FL_PUSH_BUTTON,445,35,90,20,"Histogram");
    fl_set_object_callback(obj,Graph,0);
  fdui->autobutt = obj = fl_add_lightbutton(FL_PUSH_BUTTON,445,15,90,20,"AutoBrightness");
    fl_set_object_lsize(obj,FL_TINY_SIZE);
    fl_set_object_callback(obj,automode,0);
  fdui->FD_resolution = obj = fl_add_text(FL_NORMAL_TEXT,25,15,80,20,"160x120");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->FD_bpp = obj = fl_add_text(FL_NORMAL_TEXT,25,35,80,20,"4 bpp");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
 fl_bgn_group();
  obj = fl_add_frame(FL_ENGRAVED_FRAME,140,85,560,400,"");
    fl_set_object_color(obj,FL_COL1,FL_COL1);
  fdui->FD_image1 = obj = fl_add_box(FL_FRAME_BOX,155,115,170,130,"");
  fdui->FD_image2 = obj = fl_add_box(FL_FRAME_BOX,335,115,170,130,"");
  fdui->FD_image4 = obj = fl_add_box(FL_FRAME_BOX,155,315,170,130,"");
  fdui->FD_image5 = obj = fl_add_box(FL_FRAME_BOX,335,315,170,130,"");
  obj = fl_add_text(FL_NORMAL_TEXT,155,90,50,20,"image #1");
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  obj = fl_add_text(FL_NORMAL_TEXT,335,90,50,20,"image #2");
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  obj = fl_add_text(FL_NORMAL_TEXT,155,290,50,20,"image #4");
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  obj = fl_add_text(FL_NORMAL_TEXT,335,290,50,20,"image #5");
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  obj = fl_add_text(FL_NORMAL_TEXT,230,90,95,20,"original");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
  fdui->FD_image3 = obj = fl_add_box(FL_FRAME_BOX,515,115,170,130,"");
  fdui->FD_image6 = obj = fl_add_box(FL_FRAME_BOX,515,315,170,130,"");
  obj = fl_add_text(FL_NORMAL_TEXT,515,90,50,20,"image #3");
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  obj = fl_add_text(FL_NORMAL_TEXT,515,290,50,20,"image #6");
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fl_end_group();

  obj = fl_add_text(FL_NORMAL_TEXT,35,225,65,15,"Parameters");
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->FD_PARAM_1 = obj = fl_add_valslider(FL_VERT_SLIDER,20,250,30,130,"P1");
    fl_set_object_lalign(obj,FL_ALIGN_TOP);
    fl_set_object_callback(obj,XQC_Param,0);
  fdui->FD_PARAM_2 = obj = fl_add_valslider(FL_VERT_SLIDER,50,250,30,130,"P2");
    fl_set_object_lalign(obj,FL_ALIGN_TOP);
    fl_set_object_callback(obj,XQC_Param,1);
  fdui->FD_PARAM_3 = obj = fl_add_valslider(FL_VERT_SLIDER,80,250,30,130,"P3");
    fl_set_object_lalign(obj,FL_ALIGN_TOP);
    fl_set_object_callback(obj,XQC_Param,2);
  fdui->FD_BROWSER_1 = obj = fl_add_browser(FL_HOLD_BROWSER,365,250,140,30,"");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_callback(obj,XQC_Browser,0);
  fdui->FD_DELETE_1 = obj = fl_add_button(FL_NORMAL_BUTTON,335,250,25,25,"+");
    fl_set_object_callback(obj,XQC_Button,0);
  fdui->FD_BROWSER_2 = obj = fl_add_browser(FL_HOLD_BROWSER,545,250,140,30,"");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_callback(obj,XQC_Browser,1);
  fdui->FD_BROWSER_3 = obj = fl_add_browser(FL_HOLD_BROWSER,185,450,140,30,"");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_callback(obj,XQC_Browser,2);
  fdui->FD_BROWSER_4 = obj = fl_add_browser(FL_HOLD_BROWSER,365,450,140,30,"");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_callback(obj,XQC_Browser,3);
  fdui->FD_BROWSER_5 = obj = fl_add_browser(FL_HOLD_BROWSER,545,450,140,30,"");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_callback(obj,XQC_Browser,4);
  fdui->FD_DELETE_2 = obj = fl_add_button(FL_NORMAL_BUTTON,515,250,25,25,"+");
    fl_set_object_callback(obj,XQC_Button,1);
  fdui->FD_DELETE_3 = obj = fl_add_button(FL_NORMAL_BUTTON,155,450,25,25,"+");
    fl_set_object_callback(obj,XQC_Button,2);
  fdui->FD_DELETE_4 = obj = fl_add_button(FL_NORMAL_BUTTON,335,450,25,25,"+");
    fl_set_object_callback(obj,XQC_Button,3);
  fdui->FD_DELETE_5 = obj = fl_add_button(FL_NORMAL_BUTTON,515,450,25,25,"+");
    fl_set_object_callback(obj,XQC_Button,4);
  fdui->FD_LOG_BROWSER = obj = fl_add_browser(FL_NORMAL_BROWSER,10,405,120,80,"");
  obj = fl_add_text(FL_NORMAL_TEXT,10,385,95,20,"Text window");
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->Stop = obj = fl_add_lightbutton(FL_PUSH_BUTTON,445,55,90,20,"Stop");
    fl_set_object_callback(obj,Stop,0);
  fdui->Help = obj = fl_add_button(FL_NORMAL_BUTTON,535,35,80,40,"Help");
    fl_set_object_callback(obj,Help,0);
  fdui->Save = obj = fl_add_button(FL_NORMAL_BUTTON,615,35,80,20,"Save");
    fl_set_object_callback(obj,LoadSave,1);
  fdui->Load = obj = fl_add_button(FL_NORMAL_BUTTON,615,15,80,20,"Load");
    fl_set_object_callback(obj,LoadSave,0);
  fdui->FD_graph1 = obj = fl_add_menu(FL_PULLDOWN_MENU,395,90,110,20,"operation");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE);
    fl_set_object_callback(obj,XQC_graph1,2);
  fdui->FD_graph2 = obj = fl_add_menu(FL_PULLDOWN_MENU,575,90,110,20,"operation");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE);
    fl_set_object_callback(obj,XQC_graph1,2);
  fdui->FD_graph3 = obj = fl_add_menu(FL_PULLDOWN_MENU,215,290,110,20,"operation");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE);
    fl_set_object_callback(obj,XQC_graph1,2);
  fdui->FD_graph4 = obj = fl_add_menu(FL_PULLDOWN_MENU,395,290,110,20,"operation");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE);
    fl_set_object_callback(obj,XQC_graph1,2);
  fdui->FD_graph5 = obj = fl_add_menu(FL_PULLDOWN_MENU,575,290,110,20,"operation");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE);
    fl_set_object_callback(obj,XQC_graph1,2);
  obj = fl_add_text(FL_NORMAL_TEXT,235,60,175,15,"Univ. of Western Australia");
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  obj = fl_add_text(FL_NORMAL_TEXT,230,5,170,30,"IMPROV  2.01");
    fl_set_object_boxtype(obj,FL_NO_BOX);
    fl_set_object_lsize(obj,FL_HUGE_SIZE);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(obj,FL_BOLDITALIC_STYLE+FL_SHADOW_STYLE);
  obj = fl_add_text(FL_NORMAL_TEXT,235,45,175,15,"Univ. of Stuttgart IPVR");
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  obj = fl_add_frame(FL_ENGRAVED_FRAME,225,5,190,70,"");
  fl_end_form();

  fdui->QuickCam->fdui = fdui;

  return fdui;
}
/*---------------------------------------*/

FD_AboutBox *create_form_AboutBox(void)
{
  FL_OBJECT *obj;
  FD_AboutBox *fdui = (FD_AboutBox *) fl_calloc(1, sizeof(*fdui));

  fdui->AboutBox = fl_bgn_form(FL_NO_BOX, 430, 300);
  obj = fl_add_box(FL_UP_BOX,0,0,430,300,"");
  obj = fl_add_button(FL_NORMAL_BUTTON,160,250,80,30,"OK");
    fl_set_object_callback(obj,AboutOk,0);
  fdui->AboutText = obj = fl_add_browser(FL_NORMAL_BROWSER,10,10,410,230,"");
  fl_end_form();

  fdui->AboutBox->fdui = fdui;

  return fdui;
}
/*---------------------------------------*/

FD_HGram *create_form_HGram(void)
{
  FL_OBJECT *obj;
  FD_HGram *fdui = (FD_HGram *) fl_calloc(1, sizeof(*fdui));

  fdui->HGram = fl_bgn_form(FL_NO_BOX, 400, 290);
  obj = fl_add_box(FL_UP_BOX,0,0,400,290,"");
  fdui->Histogram = obj = fl_add_xyplot(FL_FILL_XYPLOT,10,10,380,270,"");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_color(obj,FL_COL1,FL_RIGHT_BCOL);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,Histogram,0);
  fl_end_form();

  fdui->HGram->fdui = fdui;

  return fdui;
}
/*---------------------------------------*/

